/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#include <config.h>

#include <sys/stat.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <libintl.h>
#define _(a)	dgettext(PACKAGE, a)

#include "badpenguin-ansi.h"
#include "badpenguin-errors.h"
#include "badpenguin-fs.h"

#define MOVE_BUFFER_SIZE	4096


// Try to move a single file, first using the rename syscall
// and the copying the file to the target file and unlinking
// the source file.
// Keep out: the target directory should exist
int move_file( char* source, char* target) {
  int readed, writed;
  char *buffer;
  FILE *fds, *fdt;
  struct stat lpstat;
  
  // Try to rename
  if ( 0 == rename(source,target) ) return 0;
  
  // Move
  fds = fopen(source,"r");
  if (fds==NULL) {
    printERROR(_("move_file: unable to open read-only %s\n"), source);
    normal_color();
    return -1;
  }
  fdt = fopen(target,"w");
  if (fds==NULL) {
    printERROR(_("move_file: unable to open write-only %s\n"), target);
    normal_color();    
    fclose(fds);
    return -1;
  }
  
  // alloc buffer
  buffer = calloc( MOVE_BUFFER_SIZE, sizeof(char) );
  assert(buffer!=NULL);
  
  while( ! feof(fds) ) {
    readed = fread( buffer, 1, MOVE_BUFFER_SIZE, fds );
    writed = fwrite( buffer, 1, readed, fdt );
    if ( readed != writed ) {
      printERROR(_("move_file: I/O error: read %i, write %i\n"), readed, writed);
      normal_color();      
      fclose(fds);
      fclose(fdt);
      free(buffer);
      // clean up the death body :-)
      unlink(target);
      return -1;
    }
  }
  fclose(fds);
  fclose(fdt);
  free(buffer);  
  // apply
  lstat( source, &lpstat );
  if ( 0 != file_time( target, lpstat.st_mtime, lpstat.st_atime ) ) {
    printERROR(_("move_file: unable to change access and/or modification times for %s\n"), target);
    normal_color();      
  }  
  if ( 0 != chmod( target, lpstat.st_mode & 0xFFF ) ) {
    printERROR(_("move_file: unable to change permissions for %s\n"), target);
    normal_color();
  }
  if ( 0 != chown( target, lpstat.st_uid, lpstat.st_gid )) {
    printERROR(_("move_file: unable to set UID and/or GID for %s\n"), target);
    normal_color();        
  }
  if ( 0 != unlink(source) ) {
    printERROR(_("move_file: unable to remove old file for %s\n"), target);
    normal_color();        
    return -1;
  }
  return 0;
}

