/*  libbadpenguin -- generic library for Bad Penguin development
 *
 *  Copyright (C) 1996 Antonio Gallo
 *  agx@linux.it
 *  
 *  Newer versions should be on www.badpenguin.org
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  
 */

#ifdef LINUX
#include <grp.h>
#endif
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>

#include "badpenguin-ansi.h"
#include "badpenguin-chain.h"
#include "badpenguin-fs.h"
#include "badpenguin-untar.h"
#include "gnutar.h"
#include "untar-namecache.h"
#include "untar-support.h"

// This is a cache of Name and Group permissions
// Permissions are stored in tar into text and numeric format,
// because the numeric (uid & gid) can change between two systems,
// depending on the passwd and group files, we lookup for the value
// of the selected user or group into the corrispondending file
// we cache the value into thos single list to avoid going on the FS
// every time we look-up for a name. If the user exist we use the value
// from the passwd file, otherwise we use the uid stored into the
// tar archive. This can be broken ?
TChain *usercache  = NULL;
TChain *groupcache = NULL;

int match_passwd(char *s, char *line) {
  int l=strlen(s);
  if ( 0 == strncmp( s, line, l )) {
    if (line[l]==':') return 0;
  }  
  return -1;
}

int match_uid(char *line) {
  int c=0;
  int i=0;
  int l1=0;
  int l=strlen(line);
  while ((i<l) && (c<2)) {
    while ((i<l) && (line[i]!=':')) {
      i++;
    }
    c++;
    i++;
  }
  l1=i;
  while ((i<l) && (line[i]!=':')) i++;
  line[i]=0;
  line[i]=0;
  return atoi( line + l1 );
}

int match_gid(char *line) {
  int c=0;
  int i=0;
  int l1=0;
  int l=strlen(line);
  while ((i<l) && (c<3)) {
    while ((i<l) && (line[i]!=':')) {
      i++;
    }
    c++;
    i++;
  }
  l1=i;
  while ((i<l) && (line[i]!=':')) i++;
  line[i]=0;
  return atoi( line + l1 );
}

int lookup_user( char *s, TTarBlock *buffer ) {
  TChainItem *item = NULL;
  char tempstr[256];
  FILE *fd = NULL;
      
  // First, look-up the name into the cache    
  if (usercache != NULL ) {
    item = usercache->head;
    while (item!=NULL) {
      if ( 0 == strcmp(s,item->name)) {
        return TCACHEDNAME(item->data)->uid;
      }
      item = item->next;
    }
  }
  
  // Not found, look up into /etc/passwd
  snprintf( tempstr, 256, "%s/etc/passwd", gsRootDir );
  fd = fopen( tempstr, "r" );
  if (fd==NULL) {
    if (giVerbose>0) {
      printWARNING("untar: impossibile aprire %s in lettura\n", tempstr );
      normal_color();
    }
    // Cache the value
    if (usercache==NULL) usercache = chain_create();
    item = chain_append(usercache);
    item->data = calloc( 1, sizeof(TCachedName) );
    assert(item->data!=NULL);
    item->name = strdup(s);
    assert(item->name!=NULL);
    TCACHEDNAME(item->data)->gid = oct2int(buffer->gid,8);
    TCACHEDNAME(item->data)->uid = oct2int(buffer->uid,8);
    return TCACHEDNAME(item->data)->uid;
  }

  while ( ! feof(fd) ) {
    read_line(fd, tempstr);
    if ( 0 == match_passwd(s, tempstr)) {
      // Cache the entry
      if (usercache==NULL) usercache = chain_create();
      item = chain_append(usercache);
      item->data = calloc( 1, sizeof(TCachedName) );
      assert(item->data!=NULL);
      item->name = strdup(s);
      assert(item->name!=NULL);
      TCACHEDNAME(item->data)->gid = match_gid(tempstr);
      TCACHEDNAME(item->data)->uid = match_uid(tempstr);
      fclose(fd);
      return TCACHEDNAME(item->data)->uid;
    }
  }
  fclose(fd);

  printWARNING("untar: namecache has not found the user %s, with UID=%li on file %s \n", s, oct2int(buffer->uid,8), buffer->name );
  normal_color();

  // Cache the value
  if (usercache==NULL) usercache = chain_create();
  item = chain_append(usercache);
  item->data = calloc( 1, sizeof(TCachedName) );
  assert(item->data!=NULL);
  item->name = strdup(s);
  assert(item->name!=NULL);
  TCACHEDNAME(item->data)->gid = oct2int(buffer->gid,8);
  TCACHEDNAME(item->data)->uid = oct2int(buffer->uid,8);
  return TCACHEDNAME(item->data)->uid;
}

// Same as look-up user
int lookup_group( char *s, TTarBlock *buffer ){
  TChainItem *item = NULL;
  char tempstr[256];
  FILE *fd = NULL;
      
  // First, look-up the name into the cache    
  if (groupcache != NULL ) {
    item = groupcache->head;
    while (item!=NULL) {
      if ( 0 == strcmp(s,item->name)) {
        return TCACHEDNAME(item->data)->gid;
      }
      item = item->next;
    }
  }
  
  // Not found, look up into /etc/passwd
  snprintf( tempstr, 256, "%s/etc/group", gsRootDir );
  fd = fopen( tempstr, "r" );
  if (fd==NULL) {
    if (giVerbose>0) {
      printWARNING("untar: impossibile aprire %s in lettura\n", tempstr );
      normal_color();
    }

    // Cache the entry
    if (groupcache==NULL) groupcache = chain_create();
    item = chain_append(groupcache);
    item->data = calloc( 1, sizeof(TCachedName) );
    assert(item->data!=NULL);
    item->name = strdup(s);
    assert(item->name!=NULL);
    TCACHEDNAME(item->data)->uid = 0;
    TCACHEDNAME(item->data)->gid = oct2int(buffer->gid,8);
    return TCACHEDNAME(item->data)->gid;
  }

  while ( ! feof(fd) ) {
    read_line(fd, tempstr);
    if ( 0 == match_passwd(s, tempstr)) {
      // Cache the entry
      if (groupcache==NULL) groupcache = chain_create();
      item = chain_append(groupcache);
      item->data = calloc( 1, sizeof(TCachedName) );
      assert(item->data!=NULL);
      item->name = strdup(s);
      assert(item->name!=NULL);
      TCACHEDNAME(item->data)->uid = 0;
      TCACHEDNAME(item->data)->gid = match_uid(tempstr); //yes,UID!
      fclose(fd);
      return TCACHEDNAME(item->data)->gid;
    }
  }
  fclose(fd);

  printWARNING("untar: namecache has not found the group %s, with GID=%li on file %s \n", s, oct2int(buffer->gid,8), buffer->name );
  normal_color();

  // Cache the entry
  if (groupcache==NULL) groupcache = chain_create();
  item = chain_append(groupcache);
  item->data = calloc( 1, sizeof(TCachedName) );
  assert(item->data!=NULL);
  item->name = strdup(s);
  assert(item->name!=NULL);
  TCACHEDNAME(item->data)->uid = 0;
  TCACHEDNAME(item->data)->gid = oct2int(buffer->gid,8);
  return TCACHEDNAME(item->data)->gid;
}

